<?php

/**
 * The home manager controller for Dolyame.
 *
 */
class MspdolyameSettingsManagerController extends modExtraManagerController
{
    /** @var Dolyame $Dolyame */
    public $Dolyame;

    public function getVat()
    {
      return [
        ['none' , $this->modx->lexicon('mspdolyame.vat_none')],
        ['vat0' , $this->modx->lexicon('mspdolyame.vat_0')],
        ['vat10' , $this->modx->lexicon('mspdolyame.vat_10')],
        ['vat20' , $this->modx->lexicon('mspdolyame.vat_20')],
        ['vat110' , $this->modx->lexicon('mspdolyame.vat_110')],
        ['vat120' , $this->modx->lexicon('mspdolyame.vat_120')]
      ]; 
    }


    public function getPaymentMethod()
    {
        return [
        ['full_prepayment' , $this->modx->lexicon('mspdolyame.payment_method_full_prepayment')],
        ['full_payment' , $this->modx->lexicon('mspdolyame.payment_method_full_payment')],
      ];
    }
   
    public function getPaymentObject()
    {
        return [
        ['commodity' , $this->modx->lexicon('mspdolyame.payment_object_commodity')],
        ['excise' , $this->modx->lexicon('mspdolyame.payment_object_excise')],
        ['job' , $this->modx->lexicon('mspdolyame.payment_object_job')],
        ['service' , $this->modx->lexicon('mspdolyame.payment_object_service')],
        ['gambling_bet' , $this->modx->lexicon('mspdolyame.payment_object_gambling_bet')],
        ['gambling_prize' , $this->modx->lexicon('mspdolyame.payment_object_gambling_prize')],
        ['lottery' , $this->modx->lexicon('mspdolyame.payment_object_lottery')],
        ['lottery_prize' , $this->modx->lexicon('mspdolyame.payment_object_lottery_prize')],
        ['intellectual_activity' , $this->modx->lexicon('mspdolyame.payment_object_intellectual_activity')],
        ['object_payment' , $this->modx->lexicon('mspdolyame.payment_object_payment')],
        ['agent_commission' , $this->modx->lexicon('mspdolyame.payment_object_agent_commission')],
        ['composite' , $this->modx->lexicon('mspdolyame.payment_object_composite')],
        ['another' , $this->modx->lexicon('mspdolyame.payment_object_another')],
      ];
    }

    public function getRequestHandlers()
    {
        return [
            ['curl' , $this->modx->lexicon('mspdolyame.request_handler_curl')],
            ['file' , $this->modx->lexicon('mspdolyame.request_handler_file')],
        ];
    }
    

    /**
     *
     */
    public function initialize()
    {
        $this->Dolyame = $this->modx->getService('DolyamePage', 'DolyamePage', MODX_CORE_PATH . 'components/mspdolyame/model/');
        $properties = [
            'login',
            'password',
            'cert',
            'key',
            'prefix',
            'product_vat',
            'delivery_vat',
            'payment_method',
            'payment_object',
            'delivery_payment_object',
            'fiscalisation',
            'ffd120',
            'request_handler',
            'success_url',
            'fail_url',
        ];
        $output = [];
        foreach($properties as $key) {
            $setting = $this->modx->getObject('modSystemSetting', 'mspdolyame_'.$key);
            $output[$key] = $setting->get('value');

        }

        $output['vat_list'] = $this->getVat();
        $output['payment_method_list'] = $this->getPaymentMethod();
        $output['payment_object_list'] = $this->getPaymentObject();
        $output['request_handler_list'] = $this->getRequestHandlers();
        
        $this->Dolyame->config['dolyame_form'] = $output;

        parent::initialize();
    }


    /**
     * @return array
     */
    public function getLanguageTopics()
    {
        return ['mspdolyame:default'];
    }


    /**
     * @return bool
     */
    public function checkPermissions()
    {
        return true;
    }


    /**
     * @return null|string
     */
    public function getPageTitle()
    {
        return $this->modx->lexicon('mspdolyame');
    }


    /**
     * @return void
     */
    public function loadCustomCssJs()
    {
        $this->addCss($this->Dolyame->config['cssUrl'] . 'main.css');
        $this->addJavascript($this->Dolyame->config['jsUrl'] . 'mgr/dolyame.js');
        $this->addJavascript($this->Dolyame->config['jsUrl'] . 'mgr/widgets/settings.panel.js');
        $this->addJavascript($this->Dolyame->config['jsUrl'] . 'mgr/sections/settings.js');
        //$this->addJavascript($this->Dolyame->config['jsUrl'] . 'jquery.min.js');

        $this->addHtml('<script type="text/javascript">
        Dolyame.config = ' . json_encode($this->Dolyame->config) . ';
        Dolyame.config.connector_url = "' . $this->Dolyame->config['connectorUrl'] . '";
        Ext.onReady(function() {MODx.load({ xtype: "dolyame-page-settings"});});
        </script>');
    }


    /**
     * @return string
     */
    public function getTemplateFile()
    {
        $this->content .= '<div id="dolyame-panel-settings-div"></div>';

        return '';
    }
}