<?php

$newBasePaymentHandler = MODX_CORE_PATH . 'components/minishop2/handlers/mspaymenthandler.class.php';
$oldBasePaymentHandler = MODX_CORE_PATH . 'components/minishop2/model/minishop2/mspaymenthandler.class.php';

require_once __DIR__ . '/lib/Client.php';

if (!class_exists('msPaymentInterface')) {
	if (file_exists($newBasePaymentHandler)) {
		require_once $newBasePaymentHandler;
	} else {
		require_once $oldBasePaymentHandler;
	}
}

class Dolyame extends msPaymentHandler implements msPaymentInterface
{
	public $config;
	/** @var modX */
	public $modx;

	private $callbackUrl = '';

	const LOG_NAME = '[Dolyame]';

	public function __construct(xPDOObject $object, $config = [])
	{
		parent::__construct($object, $config);

		$this->modx = $object->xpdo;

		$this->modx->addPackage('mspdolyame', MODX_CORE_PATH . 'components/mspdolyame/model/');
		$this->modx->lexicon->load('mspdolyame:default');

		$siteUrl           = $this->modx->getOption('site_url');
		$assetsUrl         = $this->modx->getOption('assets_url') . 'components/mspdolyame/';
		$this->callbackUrl = $siteUrl . substr($assetsUrl, 1) . 'dolyame_callback.php';

		$this->config = array_merge([
			'login'       => $this->modx->getOption('mspdolyame_login'),
			'password'         => $this->modx->getOption('mspdolyame_password'),
			'cert'    => $this->modx->getOption('mspdolyame_cert'),
			'key'          => $this->modx->getOption('mspdolyame_key', null, false),
			'fiscalisation'          => $this->modx->getOption('mspdolyame_fiscalisation', null, 0),
			'product_vat' => $this->modx->getOption('mspdolyame_product_vat', null, 'none'),
			'delivery_vat' => $this->modx->getOption('mspdolyame_delivery_vat', null, 'none'),
			'payment_method'   => $this->modx->getOption('mspdolyame_payment_method', null),
			'payment_object'        => $this->modx->getOption('mspdolyame_payment_object', null),
			'delivery_payment_object'      => $this->modx->getOption('mspdolyame_delivery_payment_object', null, false),
			'ffd120'    => $this->modx->getOption('mspdolyame_ffd120', null, 0),
			'prefix' => $this->modx->getOption('mspdolyame_prefix', null, ''),
			'request_handler' => $this->modx->getOption('mspdolyame_request_handler', null, 'curl'),
			'success_url' => $this->modx->getOption('mspdolyame_success_url', null),
			'fail_url' => $this->modx->getOption('mspdolyame_fail_url', null),
		], $config);
	}

	/* @inheritdoc} */
	public function send(msOrder $order)
	{
		$link = $this->getModulePaymentLink($order);

		if ($link) {
			return $this->success('', ['redirect' => $link]);
		}

		return $this->error('', []);
	}

	/**
	 * Метод получения ссылки на оплату
	 * @param msOrder $order
	 * @return string
	 */
	public function getModulePaymentLink(msOrder $order)
	{
		$id        = $order->get('id');
		$sum       = $order->get('cost');
		$profile   = $order->getOne('UserProfile');
		$fullName = explode(' ', $profile->get('fullname'));
		$firstName = $fullName[0];
		$lastName = '';
		if (isset($fullName[1])) {
			$lastName = $fullName[1];
		}

		$data      = [
			'order'            => [
				'id'             => $this->config['prefix'] . $id,
				'amount'         => $sum,
				'prepaid_amount' => 0,
				'items'          => $this->getOrderItems($order),
			],
			'client_info'      => [
				'first_name' => $firstName,
				'last_name'  => $lastName,
				'phone'      => $this->clearPhone($profile->get('mobilephone')),
				'email'      => $profile->get('email'),
			],
			'notification_url' => $this->callbackUrl."?order_id=".$id,
			'fail_url'         => $this->config['success_url'],
			'success_url'      => $this->config['fail_url'],
		];
		if ($this->config['fiscalisation']) {
			$data['fiscalization_settings'] = ['type' => 'enabled'];
		}

		$this->info($data);


		$client   = $this->initClient();
		$response = $client->create($data);

		$this->info($response);

		$paymentSchedule = isset($response['payment_schedule']) ? $response['payment_schedule'] : null;

		$this->saveOrder(
			[
			'id'     => $data['order']['id'],
			'order_id'     => $id,
			'status'           => $response['status'],
			'items'            => json_encode($data['order']['items'], JSON_UNESCAPED_UNICODE),
			'date'             => date('Y-m-d H:i:s'),
			'amount'           => $data['order']['amount'],
			'payment_schedule' => json_encode($paymentSchedule),
			]
		);


		return $response['link'];
	}

	public function getOrderItems(msOrder $order) {
		$products = $order->getMany('Products');

		if (!$products) {
			return [];
		}
		$positions = [];

		foreach ($products as $product) {
			$productsCost = 0;
			if ((float) $paymentPrice < 0 && preg_match('/%$/', $paymentPrice)) {
				$productsCost += ($product->get('cost') - ($product->get('cost') / 100 * abs((float) $paymentPrice)));
			}
			if ((float) $paymentPrice > 0 && preg_match('/%$/', $paymentPrice)) {
				$productsCost += ($product->get('cost') + ($product->get('cost') / 100 * abs((float) $paymentPrice)));
			}

			if ((float) $paymentPrice === 0.0) {
				$productsCost += $product->get('cost');
			}
			$position = [
					'name'     => $product->get('name'),
					'quantity' => $product->get('count'),
					'price'    => round($productsCost / $product->get('count') , 2),
			];

			if ($this->config['fiscalisation']) {
				$receipt = [];
				$receipt['payment_method'] = $this->config['payment_method'];
				$receipt['tax']            = $this->config['product_vat'];
				if ($this->config['ffd120']) {
					$receipt['payment_object']   = $this->config['payment_object'];
					$receipt['measurement_unit'] = $this->modx->lexicon('mspdolyame.unit');
				}
				$position['receipt'] = $receipt;
			}

			$positions[] = $position;
		}

		if ($order->get('delivery_cost') > 0) {
			$position = [
					'name'     => $this->modx->lexicon('mspdolyame.delivery'),
					'quantity' => 1,
					'price'    => $order->get('delivery_cost'),
			];

			if ($this->config['fiscalisation']) {
				$receipt = [];
				$receipt['payment_method'] = $this->config['payment_method'];
				$receipt['tax']            = $this->config['delivery_vat'];
				if ($this->config['ffd120']) {
					$receipt['payment_object']   = $this->config['delivery_payment_object'];
					$receipt['measurement_unit'] = $this->modx->lexicon('mspdolyame.unit');
				}
				$position['receipt'] = $receipt;
			}

			$positions[] = $position;
		}
		return $positions;
	}


	public function getOrderInfo($id)
	{
		$client  = $this->initClient();

		$result = $client->info($this->config['prefix'].$id);
		return $result;
	}

	public function updateTransaction($orderId, $data)
	{
		$transaction = $this->modx->getObject(mspDolyameTransaction::class, ['order_id' => $orderId]);
		$transaction->set('status', $data['status']);
		$transaction->save();
	}

	/* @inheritdoc} */
	public function receive(msOrder $order)
	{
		$id   = $order->get('id');
		$statusPaid = $this->modx->getOption('ms2_status_paid', null, 2);
		$this->ms2->changeOrderStatus($id, $statusPaid);
		exit('OK');
	}

	/**
	 * @param $text
	 * @param array $request
	 */
	public function paymentError($text, $request = [])
	{
		$this->modx->log(
			modX::LOG_LEVEL_ERROR,
			self::LOG_NAME . ' ' . $text . ', request: ' . print_r($request, true)
		);
		header("HTTP/1.0 400 Bad Request");

		die('ERR: ' . $text);
	}

	public function info($message)
	{
		/*
		if (!$this->config['debug']) {
			return;
		}*/
		if (!is_scalar($message)) {
			$message = var_export($message, true);
		}
		$old = $this->modx->setLogLevel(3);
		$this->modx->log(3, $message, ["target" => "FILE", "options" => ["filename" => 'dolyame.log']]);
		$this->modx->setLogLevel($old);
    }

	public function saveOrder(array $saveArr)
	{

		$q = $this->modx->newObject('mspDolyameTransaction');
		$q->fromArray($saveArr);
		$q->save();

		return true;
	}

	public function clearPhone($phone)
	{
		$phone = preg_replace("#[^\d]#", "", $phone);
		if (!preg_match("#[7|8]{0,1}(\d{10})#", $phone, $match)) {
			return '';
		}
		return '+7' . $match[1];
	}

	public function confirmPayment($id, $items)
	{

		$amount = 0;
		foreach($items as $item) {
			$amount += $item['price'] * $item['quantity'];
		}

		$client  = $this->initClient();

		$data = [
			'amount'         => $amount,
			'items'          => $items,
			'prepaid_amount' => 0,
		];
		$result = $client->commit($id, $data);
		return $result;
	}

	public function refundPayment($id, $items)
	{

		$amount = 0;
		foreach($items as $item) {
			$amount += $item['price'] * $item['quantity'];
		}

		$client  = $this->initClient();

		$data = [
			'amount'         => $amount,
			'returned_items'          => $items,
			'refunded_prepaid_amount' => 0,
		];
		if ($this->config['fiscalisation']) {
			$data['fiscalization_settings'] = ['type' => 'enabled'];
		}
		$result = $client->refund($id, $data);
		return $result;
	}



	private function initClient()
	{
		$api = new \Dolyame\Payment\Client($this->config['login'], $this->config['password']);
		$api->setLogger($this);
		$certPath = $this->config['cert'];
		if (strpos($this->config['cert'], "-----BEGIN CERTIFICATE-----") === false) {
			$certPath = realpath(MODX_BASE_PATH) . '/' . ltrim($this->config['cert'], '/');
		}
		$api->setCertPath($certPath);

		$keyPath = $this->config['key'];
		if (strpos($keyPath, "--") === false ){
			$keyPath = realpath(MODX_BASE_PATH) . '/' . ltrim($this->config['key'], '/');
		}
		$api->setKeyPath($keyPath);
		if ($this->config['request_handler'] === 'file') {
			$api->useFileRequestHandler();
		}
		return $api;
	}
}
