<?php

class MspDolyameAcceptProcessor extends modObjectProcessor
{
    public $objectType = 'mspDolyameTransaction';
    public $classKey = 'mspDolyameTransaction';
    public $defaultSortField = 'id';
    public $defaultSortDirection = 'DESC';
    public $languageTopics = array('mspdolyame:transactions');


    /**
    * @return bool|null|string
    */
    public function initialize()
    {
        $primaryKey = $this->getProperty($this->primaryKeyField,false);
        if (empty($primaryKey)) return $this->modx->lexicon($this->objectType.'_err_ns');
        $this->object = $this->modx->getObject($this->classKey,$primaryKey);
        if (empty($this->object)) return $this->modx->lexicon($this->objectType.'_err_nfs',array($this->primaryKeyField => $primaryKey));

        if ($this->checkViewPermission && $this->object instanceof modAccessibleObject && !$this->object->checkPolicy('view')) {
            return $this->modx->lexicon('access_denied');
        }

        return parent::initialize();

    }

    public function process()
    {
        $items = $this->getProperty('items');
        $id = $this->getProperty($this->primaryKeyField,false);
        $miniShop2 = $this->modx->getService('minishop2');
        $miniShop2->loadCustomClasses('payment');
        $handler = new Dolyame($this->modx->newObject(msPayment::class));

        $items = json_decode($items, true);

        $fiscalisation = $this->modx->getOption('mspdolyame_fiscalisation', null, 0);

        foreach($items as &$item) {
            unset($item['id']);
            if (!$fiscalisation) {
                unset($item['receipt']);
            }
        }

        try{
            $result = $handler->confirmPayment($id, $items);
            $this->object->set('items', json_encode($items));
            $this->object->set('status', 'committed');
            $this->object->set('amount', $result['amount']);
            $this->object->save();
        } catch (Exception $e){
            return $this->failure($this->modx->lexicon('dolyame_accept_error').$e->getMessage());
        }

        $this->logManagerAction();

        return $this->success('',array($this->primaryKeyField => $this->object->get($this->primaryKeyField)));
    }

    public function logManagerAction() {
        $this->modx->logManagerAction($this->objectType.'_accept',$this->classKey,$this->object->get($this->primaryKeyField));
    }
}

return 'MspDolyameAcceptProcessor';
