<?php

class MspDolyameGetListProcessor extends modObjectGetListProcessor
{
    public $objectType = 'mspDolyameTransaction';
    public $classKey = 'mspDolyameTransaction';
    public $defaultSortField = 'id';
    public $defaultSortDirection = 'DESC';

    public function process() {
      
      $properties = [
        'login' => trim($this->getProperty('login')),
        'password' => trim($this->getProperty('password')),
        'cert' => trim($this->getProperty('cert')),
        'key' => trim($this->getProperty('key')),
        'prefix' => trim($this->getProperty('prefix')),
        'product_vat' => trim($this->getProperty('product_vat')),
        'delivery_vat' => trim($this->getProperty('delivery_vat')),
        'payment_method' => trim($this->getProperty('payment_method')),
        'payment_object' => trim($this->getProperty('payment_object')),
        'delivery_payment_object' => trim($this->getProperty('delivery_payment_object')),
        'fiscalisation' => trim($this->getProperty('fiscalisation')),
        'ffd120' => trim($this->getProperty('ffd120')),
        'request_handler' => trim($this->getProperty('request_handler')),
        'success_url' => trim($this->getProperty('success_url')),
        'fail_url' => trim($this->getProperty('fail_url')),
      ];

      foreach($properties as $key => $value) {
        $setting = $this->modx->getObject('modSystemSetting', 'mspdolyame_'.$key);
        $setting->set('value', $value);
        $setting->save();
      }
      
      $cacheRefreshOptions =  [
        'system_settings' => [],
        'resource' => [],
      ];
      $this->modx->cacheManager->refresh($cacheRefreshOptions);

      return $this->success('ok');
    }
    
    /**
     * We do a special check of permissions
     * because our objects is not an instances of modAccessibleObject
     *
     * @return boolean|string
     */
    public function beforeQuery()
    {
        if (!$this->checkPermissions()) {
            return $this->modx->lexicon('access_denied');
        }

        return true;
    }


    /**
     * @param xPDOQuery $c
     *
     * @return xPDOQuery
     */
    public function prepareQueryBeforeCount(xPDOQuery $c)
    {
        $query = trim($this->getProperty('query'));
        if ($query) {
            $c->where([
                'name:LIKE' => "%{$query}%",
                'OR:description:LIKE' => "%{$query}%",
            ]);
        }

        return $c;
    }

}

return 'MspDolyameGetListProcessor';