<?php

class MspDolyameGetTransactionsProcessor extends modObjectGetListProcessor
{
    public $objectType = 'mspDolyameTransaction';
    public $classKey = 'mspDolyameTransaction';
    public $defaultSortField = 'id';
    public $defaultSortDirection = 'DESC';
    public $languageTopics = array('mspdolyame:transactions');

    
    /**
     * We do a special check of permissions
     * because our objects is not an instances of modAccessibleObject
     *
     * @return boolean|string
     */
    public function beforeQuery()
    {
        if (!$this->checkPermissions()) {
            return $this->modx->lexicon('access_denied');
        }

        return true;
    }


    /**
     * @param xPDOQuery $c
     *
     * @return xPDOQuery
     */
    public function prepareQueryBeforeCount(xPDOQuery $c)
    {
        $query = trim($this->getProperty('query'));
        if ($query) {
            $c->where([
                'order_id:LIKE' => "%{$query}%",
                //'OR:description:LIKE' => "%{$query}%",
            ]);
        }

        return $c;
    }


    /**
     * @param xPDOObject $object
     *
     * @return array
     */
    public function prepareRow(xPDOObject $object)
    {
        $array = $object->toArray();
        $array['actions'] = [];

        if ($array['status'] == 'waiting_for_commit') {
            $array['actions'][] = [
                'cls' => 'x-btn x-btn-small x-btn-icon-small-left primary-button x-btn-noicon',
                'icon' => 'icon',
                'title' => $this->modx->lexicon('dolyame_accept_button'),
                'action' => 'acceptTransaction',
                'button' => true,
                'menu' => true,
            ];
        }

        if ($array['status'] == 'wait_for_commit') {
            $array['actions'][] = [
                'cls' => 'x-btn x-btn-small x-btn-icon-small-left primary-button x-btn-noicon',
                'icon' => 'icon',
                'title' => $this->modx->lexicon('dolyame_accept_button'),
                'action' => 'acceptTransaction',
                'button' => true,
                'menu' => true,
            ];
        }

        if ($array['status'] == 'committed') {
            $array['actions'][] = [
                'cls' => 'x-btn x-btn-small x-btn-icon-small-left x-btn-noicon',
                'icon' => 'icon',
                'title' => $this->modx->lexicon('dolyame_refund_button'),
                'action' => 'refundTransaction',
                'button' => true,
                'menu' => true,
            ];
        }
        $array['status'] = $this->modx->lexicon('dolyame_status_'.$array['status']);
        return $array;
    }

}

return 'MspDolyameGetTransactionsProcessor';